# -*- coding: utf-8 -*-

"""
Package setup.

Set me up with `python setup.py bdist_wheel --universal`
"""
import io
import json
import os

from setuptools import setup

# Get long description from readme
with io.open("README.md", "rt", encoding="utf8") as readmefile:
    README = readmefile.read()

# Convert emojilib json file to python dictionary, so we don't have the parse
# penalty at runtime

# The dir where this file is located is relative to the root dir (where the
# tox.ini file is located). Our tox setup will inject an env variable with that
# path, because we do isolated tox builds in non-root folders eg when running in
# parallel
SETUP_RUN_DIR = os.environ.get("TOX_INI_DIR", os.path.dirname(__file__))


def generate_emoji_db(root_dir, outfile="emoji_fzf/emoji_fzf_emojilib.py"):
    # Load the gemoji database as a baseline
    with io.open(
        os.path.join(root_dir, "gemoji/db/emoji.json"),
        "rt",
        encoding="utf8",
    ) as gemoji_db:
        GEMOJI_DB_DATA = json.load(gemoji_db)

    EMOJI_DICT = {}

    # create the dictionary of emojis, keyed by canonical name
    for emoji in GEMOJI_DB_DATA:
        # the emoji canonical name; can contain spaces, replace with _
        NAME = "_".join(emoji["description"].split(" "))
        EMOJI_DICT[NAME] = {
            # the unicode codepoint sequence itself
            "emoji": emoji["emoji"],
            # any aliases the gemoji lib provides
            "aliases": tuple(emoji["aliases"]),
        }

    # merge the aliases from the emojilib database
    # 1. load the json
    with io.open(
        os.path.join(root_dir, "emojilib/dist/emoji-en-US.json"),
        "rt",
        encoding="utf8",
    ) as emojilib_db:
        EMOJILIB_DB_DATA = json.load(emojilib_db)
    # 2. dictionary, keyed by codepoint, of aliases
    EMOJILIB_DB_BY_CODEPOINT = {}
    for codepoint, aliases in EMOJILIB_DB_DATA.items():
        EMOJILIB_DB_BY_CODEPOINT[codepoint] = tuple(aliases)
    # 3. iterate over the gemoji dictionary, updating any common codepoint with the
    #    aliases in the emojilib dictionary. use set() to eliminate duplicates (no
    #    lexical de-duping tho)
    for key, val in EMOJI_DICT.items():
        additional_aliases = EMOJILIB_DB_BY_CODEPOINT.pop(val["emoji"], tuple())
        EMOJI_DICT[key]["aliases"] = tuple(set(val["aliases"] + additional_aliases))

    # 4. add in any emojilib entries that weren't in the gemoji database
    for codepoint, (name, *aliases) in EMOJILIB_DB_BY_CODEPOINT.items():
        # some of these have spaces, sub in '_' to make it work nicer with fzf
        # preview
        name = name.replace(" ", "_")
        EMOJI_DICT[name] = {"emoji" : codepoint, "aliases": tuple(aliases)}

    with open(outfile, "w") as genfile:
        genfile.write(
            """# -*- coding: utf-8 -*-
\"\"\"
Emoji python alias file, autogenerated.
\"\"\"

"""
        )

        # Unfortunately this doesn't work; we end up with literals like '\\U0001f600' 😕
        # not suitable for printing alas. For now, just archive the py2 output,
        # which on both
        #
        # # we want the emojilib file to have the literal symbols represented as ascii
        # # literals with \u escape sequences, which we can safely use in python2 with
        # # codecs.getwriter('utf-8') to print the symbol. to make sure the emojilib
        # # is formatted correctly when the wheel is built with python3, use a little
        # # goofy logic.
        # import sys
        # if sys.version_info >= (3,):
        #     for key, val in EMOJI_DICT.items():
        #         EMOJI_DICT[key]["emoji"] = (
        #             val["emoji"].encode("ascii", "backslashreplace").decode("utf-8")
        #         )

        genfile.write("EMOJIS = {}".format(EMOJI_DICT))


if __name__ == "__main__":
    generate_emoji_db(SETUP_RUN_DIR)

    setup(
        # I think using `-` instead of `_` is more user-friendly, but due to python
        # import directives not allowing `-`, keep everything consistent with `_`.
        name="emoji-fzf",
        version="0.7.0",
        description="Emoji searcher for use with fzf",
        author="Noah Pendleton",
        author_email="2538614+noahp@users.noreply.github.com",
        url="https://github.com/noahp/emoji-fzf",
        project_urls={
            "Code": "https://github.com/noahp/emoji-fzf",
            "Issue tracker": "https://github.com/noahp/emoji-fzf/issues",
        },
        long_description=README,
        long_description_content_type="text/markdown",
        install_requires=["click"],
        # using markdown as pypi description:
        # https://dustingram.com/articles/2018/03/16/markdown-descriptions-on-pypi
        setup_requires=["setuptools>=38.6.0", "wheel>=0.31.0", "twine>=1.11.0"],
        packages=["emoji_fzf"],
        entry_points={"console_scripts": ["emoji-fzf = emoji_fzf.emoji_fzf:cli"]},
        classifiers=[
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Operating System :: OS Independent",
        ],
    )
